## $Id: browser.py 15346 2019-03-06 22:19:56Z henrik $
##
## Copyright (C) 2012 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
from datetime import datetime
import httplib
import urllib
import urllib2
import re
from xml.dom.minidom import parseString
import grok
from zope.component import getUtility
from zope.catalog.interfaces import ICatalog
from waeup.kofa.interfaces import IUniversity, CLEARED
from waeup.kofa.payments.interfaces import IPayer
from waeup.kofa.webservices import PaymentDataWebservice
from waeup.kofa.browser.layout import KofaPage, UtilityView
from waeup.kofa.students.viewlets import ApprovePaymentActionButton as APABStudent
from waeup.kofa.applicants.viewlets import ApprovePaymentActionButton as APABApplicant
from kofacustom.nigeria.interswitch.browser import (
    InterswitchActionButtonStudent,
    InterswitchRequestWebserviceActionButtonStudent,
    InterswitchActionButtonApplicant,
    InterswitchRequestWebserviceActionButtonApplicant)
from kofacustom.nigeria.interfaces import MessageFactory as _
from kofacustom.nigeria.students.interfaces import INigeriaStudentOnlinePayment
from kofacustom.nigeria.applicants.interfaces import INigeriaApplicantOnlinePayment
from kofacustom.nigeria.etranzact.tests import PAYOUTLET_QUERY_URL, TERMINAL_ID

grok.templatedir('browser_templates')

def module_activated(session):
    try:
        return getattr(grok.getSite()['configuration'][str(session)],
            'etranzact_payoutlet_enabled', False)
    except KeyError:
        return False

# Requerying Etranzact Payoutlet payments

def query_payoutlet(confirmation_number, payment, terminal_id, query_url):
    postdict = {}
    postdict['TERMINAL_ID'] = terminal_id
    postdict['RESPONSE_URL'] = 'http://dummy'
    postdict['CONFIRMATION_NO'] = confirmation_number
    data = urllib.urlencode(postdict)
    payment.conf_number = confirmation_number
    try:
        # Etranzact only accepts HTTP 1.1 requests. Therefore
        # the urllib2 package is required here.
        f = urllib2.urlopen(url=query_url, data=data)
        success = f.read()
        success = success.replace('\r\n','')
        # Etranzact sends strange HTML tags which must be removed.
        success = re.sub("<.*?>", "", success)
        if 'CUSTOMER_ID' not in success:
            msg = _('Invalid or unsuccessful callback: ${a}',
                mapping = {'a': success})
            log = 'invalid callback for payment %s: %s' % (payment.p_id, success)
            payment.p_state = 'failed'
            return False, msg, log
        success = success.replace('%20',' ').split('&')
        # We expect at least two parameters
        if len(success) < 2:
            msg = _('Invalid callback: ${a}', mapping = {'a': success})
            log = 'invalid callback for payment %s: %s' % (payment.p_id, success)
            payment.p_state = 'failed'
            return False, msg, log
        try:
            success_dict = dict([tuple(i.split('=')) for i in success])
        except ValueError:
            msg = _('Invalid callback: ${a}', mapping = {'a': success})
            log = 'invalid callback for payment %s: %s' % (payment.p_id, success)
            payment.p_state = 'failed'
            return False, msg, log
    except IOError:
        msg = _('Etranzact IOError')
        log = 'Etranzact IOError'
        return False, msg, log
    payment.r_code = u'ET'
    payment.r_company = u'etranzact'
    payment.r_desc = u'%s' % success_dict.get('TRANS_DESCR')
    payment.r_amount_approved = float(success_dict.get('TRANS_AMOUNT',0.0))
    payment.r_card_num = None
    payment.r_pay_reference = u'%s' % success_dict.get('RECEIPT_NO')
    if payment.r_amount_approved != payment.amount_auth:
        msg = _('Wrong amount')
        log = 'wrong callback for payment %s: %s' % (payment.p_id, success)
        payment.p_state = 'failed'
        return False, msg, log
    customer_id = success_dict.get('CUSTOMER_ID')
    if payment.p_id != customer_id:
        msg = _('Wrong payment id')
        log = 'wrong callback for payment %s: %s' % (payment.p_id, success)
        payment.p_state = 'failed'
        return False, msg, log
    log = 'valid callback for payment %s: %s' % (payment.p_id, success)
    msg = _('Successful callback received')
    payment.p_state = 'paid'
    payment.payment_date = datetime.utcnow()
    return True, msg, log

class EtranzactEnterPinActionButtonApplicant(APABApplicant):
    grok.context(INigeriaApplicantOnlinePayment)
    grok.require('waeup.payApplicant')
    grok.order(3)
    icon = 'actionicon_call.png'
    text = _('Enter Etranzact PIN')
    target = 'enterpin'

    @property
    def target_url(self):
        if not module_activated(self.context.__parent__.__parent__.year):
            return ''
        if self.context.p_state != 'unpaid':
            return ''
        return self.view.url(self.view.context, self.target)

class EtranzactEnterPinActionButtonStudent(APABStudent):
    grok.context(INigeriaStudentOnlinePayment)
    grok.require('waeup.payStudent')
    grok.order(3)
    icon = 'actionicon_call.png'
    text = _('Enter Etranzact PIN')
    target = 'enterpin'

    @property
    def target_url(self):
        if not module_activated(self.context.student.current_session):
            return ''
        if self.context.p_state != 'unpaid':
            return ''
        return self.view.url(self.view.context, self.target)

class EtranzactEnterPinPageStudent(KofaPage):
    """
    """
    grok.context(INigeriaStudentOnlinePayment)
    grok.name('enterpin')
    grok.template('enterpin')
    grok.require('waeup.payStudent')

    buttonname = _('Submit to Etranzact')
    label = _('Requery Etranzact History')
    action = 'query_payoutlet_history'
    placeholder = _('Confirmation Number (PIN)')

    def update(self):
        if not module_activated(self.context.student.current_session):
            return
        super(EtranzactEnterPinPageStudent, self).update()
        return

class EtranzactEnterPinPageApplicant(EtranzactEnterPinPageStudent):
    """
    """
    grok.require('waeup.payApplicant')
    grok.context(INigeriaApplicantOnlinePayment)

class EtranzactQueryHistoryPageStudent(UtilityView, grok.View):
    """ Query history of Etranzact payments
    """
    grok.context(INigeriaStudentOnlinePayment)
    grok.name('query_payoutlet_history')
    grok.require('waeup.payStudent')
    terminal_id = TERMINAL_ID
    query_url = PAYOUTLET_QUERY_URL

    def update(self, confirmation_number=None):
        if not module_activated(self.context.student.current_session):
            return
        if self.context.p_state == 'paid':
            self.flash(_('This ticket has already been paid.'))
            return
        student = self.context.student
        success, msg, log = query_payoutlet(
            confirmation_number,self.context, self.terminal_id, self.query_url)
        student.writeLogMessage(self, log)
        if not success:
            self.flash(msg)
            return
        flashtype, msg, log = self.context.doAfterStudentPayment()
        if log is not None:
            student.writeLogMessage(self, log)
        self.flash(msg, type=flashtype)
        return

    def render(self):
        self.redirect(self.url(self.context, '@@index'))
        return

class EtranzactQueryHistoryPageApplicant(UtilityView, grok.View):
    """ Query history of Etranzact payments
    """
    grok.context(INigeriaApplicantOnlinePayment)
    grok.name('query_payoutlet_history')
    grok.require('waeup.payApplicant')
    terminal_id = TERMINAL_ID
    query_url = PAYOUTLET_QUERY_URL

    def update(self, confirmation_number=None):
        if not module_activated(self.context.__parent__.__parent__.year):
            return
        ob_class = self.__implemented__.__name__
        if self.context.p_state == 'paid':
            self.flash(_('This ticket has already been paid.'))
            return
        applicant = self.context.__parent__
        success, msg, log = query_payoutlet(
            confirmation_number,self.context, self.terminal_id, self.query_url)
        applicant.writeLogMessage(self, log)
        if not success:
            self.flash(msg)
            return
        flashtype, msg, log = self.context.doAfterApplicantPayment()
        if log is not None:
            applicant.writeLogMessage(self, log)
        self.flash(msg, type=flashtype)
        return

    def render(self):
        self.redirect(self.url(self.context, '@@index'))
        return